
if { $argc != 1 } {
	puts "The script requires the project name to be set as an argument."	
} else {

	start_gui
	
	set proj_name [lindex $argv 0]
	set script_path [ file dirname [ file normalize [ info script ] ] ]
	set vivado_dir_path ../vivado
	set ip_repo_dir_path ../ix_repo
	set hdl_dir_path ../hdl
	set constraint_dir_path ../constraints

	create_project $proj_name $vivado_dir_path/$proj_name -part xc7z030fbg676-3

	set_property target_language VHDL [current_project]
	
	# Work-around suggested by Xilinx to avoid the following inconsequential critical warning
	# [Memdata 28-165] The reference name: top_i_ip_m_blk_mem_gen_0 was not found in a previous reference definition. Either the bmm file or the bmm_info_* properties are malformed, therefore BRAM INIT strings can not be populated.
	# Source: https://support.xilinx.com/s/article/59639?language=en_US
	set_param memdata.disableAnnotatedBMM 1
	set_param memdata.disableBRAMInitStrings 1
	set_param memdata.ignoreEmbeddedBMMProperties 1

	set_property  ip_repo_paths  $ip_repo_dir_path [current_project]
	update_ip_catalog
	create_bd_design "top"
	create_bd_cell -type ip -vlnv imperix.ch:ix:IXIP:3.10 IXIP_0
	set_property name IXIP [get_bd_cells IXIP_0]

	add_files -fileset constrs_1 -norecurse $constraint_dir_path/sandbox_pins_3.10.xdc

	make_bd_intf_pins_external  [get_bd_intf_pins IXIP/FIXED_IO]
	make_bd_intf_pins_external  [get_bd_intf_pins IXIP/DDR]
	make_bd_intf_pins_external  [get_bd_intf_pins IXIP/BBOX]
	make_bd_intf_pins_external  [get_bd_intf_pins IXIP/USR]

	set_property name FIXED_IO [get_bd_intf_ports FIXED_IO_0]
	set_property name DDR [get_bd_intf_ports DDR_0]
	set_property name BBOX [get_bd_intf_ports BBOX_0]
	set_property name USR [get_bd_intf_ports USR_0]

	make_bd_pins_external  [get_bd_pins IXIP/private_out]
	make_bd_pins_external  [get_bd_pins IXIP/pwm]
	make_bd_pins_external  [get_bd_pins IXIP/gpo]
	make_bd_pins_external  [get_bd_pins IXIP/private_in]
	make_bd_pins_external  [get_bd_pins IXIP/flt]
	make_bd_pins_external  [get_bd_pins IXIP/gpi]

	set_property name private_out [get_bd_ports private_out_0]
	set_property name pwm [get_bd_ports pwm_0]
	set_property name gpo [get_bd_ports gpo_0]
	set_property name private_in [get_bd_ports private_in_0]
	set_property name flt [get_bd_ports flt_0]
	set_property name gpi [get_bd_ports gpi_0]

	create_bd_cell -type ip -vlnv xilinx.com:ip:xlconstant:1.1 xlconstant_0
	set_property name USER_ID [get_bd_cells xlconstant_0]
	set_property -dict [list CONFIG.CONST_WIDTH {16}] [get_bd_cells USER_ID]
	set_property -dict [list CONFIG.CONST_VAL {1}] [get_bd_cells USER_ID]
	connect_bd_net [get_bd_pins USER_ID/dout] [get_bd_pins IXIP/user_fw_id]

	make_wrapper -files [get_files $vivado_dir_path/$proj_name/$proj_name.srcs/sources_1/bd/top/top.bd] -top
	import_files -force -norecurse $vivado_dir_path/$proj_name/$proj_name.gen/sources_1/bd/top/hdl/top_wrapper.vhd

	add_files -norecurse $hdl_dir_path/AXIS_interface.vhd
	create_bd_cell -type module -reference ix_axis_interface ix_axis_interface_0
	set_property name ix_axis_interface [get_bd_cells ix_axis_interface_0]
	connect_bd_intf_net [get_bd_intf_pins IXIP/SBIO_BUS] [get_bd_intf_pins ix_axis_interface/SBIO_BUS]
	connect_bd_intf_net [get_bd_intf_pins IXIP/ADC] [get_bd_intf_pins ix_axis_interface/ADC]
	connect_bd_net [get_bd_pins IXIP/adc_done_pulse] [get_bd_pins ix_axis_interface/adc_done_pulse_in]
	connect_bd_net [get_bd_pins IXIP/data_valid_pulse] [get_bd_pins ix_axis_interface/data_valid_pulse_in]
	connect_bd_net [get_bd_pins IXIP/reading] [get_bd_pins ix_axis_interface/data_reading_in]
	connect_bd_net [get_bd_pins IXIP/clk_250_mhz] [get_bd_pins ix_axis_interface/clk]
	connect_bd_net [get_bd_pins IXIP/sync_pulse] [get_bd_pins ix_axis_interface/sync_pulse_in]
	
	# Configure IXIP
	set_property -dict [list CONFIG.SB_GT_SFP_0 {true} CONFIG.SB_GT_SFP_1 {true}] [get_bd_cells IXIP]
	
	# Instantiate Aurora 0
	create_bd_cell -type ip -vlnv xilinx.com:ip:aurora_8b10b:11.1 aurora_8b10b_0
	connect_bd_net [get_bd_pins IXIP/rxn_0] [get_bd_pins aurora_8b10b_0/rxn]
	connect_bd_net [get_bd_pins IXIP/rxp_0] [get_bd_pins aurora_8b10b_0/rxp]
	connect_bd_net [get_bd_pins aurora_8b10b_0/txn] [get_bd_pins IXIP/txn_0]
	connect_bd_net [get_bd_pins aurora_8b10b_0/txp] [get_bd_pins IXIP/txp_0]
	connect_bd_net [get_bd_pins aurora_8b10b_0/gt0_qplllock_in] [get_bd_pins IXIP/GT_gt0_qplllock]
	connect_bd_net [get_bd_pins aurora_8b10b_0/gt0_qpllrefclklost_in] [get_bd_pins IXIP/GT_gt0_qpllrefclklost]
	connect_bd_net [get_bd_pins IXIP/GT_gt_qpllclk_quad1] [get_bd_pins aurora_8b10b_0/gt_qpllclk_quad1_in]
	connect_bd_net [get_bd_pins aurora_8b10b_0/gt_qpllrefclk_quad1_in] [get_bd_pins IXIP/GT_gt_qpllrefclk_quad1]
	connect_bd_net [get_bd_pins IXIP/GT_gt_refclk1] [get_bd_pins aurora_8b10b_0/gt_refclk1]
	connect_bd_net [get_bd_pins aurora_8b10b_0/init_clk_in] [get_bd_pins IXIP/clk_50_mhz]
	set_property -dict [list CONFIG.C_LANE_WIDTH {4} CONFIG.C_REFCLK_FREQUENCY {250.000} CONFIG.Interface_Mode {Streaming}] [get_bd_cells aurora_8b10b_0]
	connect_bd_net [get_bd_pins aurora_8b10b_0/drpclk_in] [get_bd_pins IXIP/clk_50_mhz]
	
	# Instantiate BUFG
	create_bd_cell -type ip -vlnv xilinx.com:ip:util_ds_buf:2.2 util_ds_buf_0
	set_property -dict [list CONFIG.C_BUF_TYPE {BUFG}] [get_bd_cells util_ds_buf_0]
	connect_bd_net [get_bd_pins util_ds_buf_0/BUFG_I] [get_bd_pins aurora_8b10b_0/tx_out_clk]
	connect_bd_net [get_bd_pins util_ds_buf_0/BUFG_O] [get_bd_pins aurora_8b10b_0/user_clk]
	connect_bd_net [get_bd_pins aurora_8b10b_0/sync_clk] [get_bd_pins util_ds_buf_0/BUFG_O]
	
	# Instantiate Aurora 1
	create_bd_cell -type ip -vlnv xilinx.com:ip:aurora_8b10b:11.1 aurora_8b10b_1
	connect_bd_net [get_bd_pins aurora_8b10b_1/rxn] [get_bd_pins IXIP/rxn_1]
	connect_bd_net [get_bd_pins aurora_8b10b_1/rxp] [get_bd_pins IXIP/rxp_1]
	connect_bd_net [get_bd_pins aurora_8b10b_1/txn] [get_bd_pins IXIP/txn_1]
	connect_bd_net [get_bd_pins aurora_8b10b_1/txp] [get_bd_pins IXIP/txp_1]
	connect_bd_net [get_bd_pins aurora_8b10b_1/gt0_qplllock_in] [get_bd_pins IXIP/GT_gt0_qplllock]
	connect_bd_net [get_bd_pins aurora_8b10b_1/gt0_qpllrefclklost_in] [get_bd_pins IXIP/GT_gt0_qpllrefclklost]
	connect_bd_net [get_bd_pins aurora_8b10b_1/gt_qpllclk_quad1_in] [get_bd_pins IXIP/GT_gt_qpllclk_quad1]
	connect_bd_net [get_bd_pins aurora_8b10b_1/gt_qpllrefclk_quad1_in] [get_bd_pins IXIP/GT_gt_qpllrefclk_quad1]
	connect_bd_net [get_bd_pins aurora_8b10b_1/gt_refclk1] [get_bd_pins IXIP/GT_gt_refclk1]
	connect_bd_net [get_bd_pins aurora_8b10b_1/init_clk_in] [get_bd_pins IXIP/clk_50_mhz]
	connect_bd_net [get_bd_pins aurora_8b10b_1/user_clk] [get_bd_pins util_ds_buf_0/BUFG_O]
	connect_bd_net [get_bd_pins aurora_8b10b_1/sync_clk] [get_bd_pins util_ds_buf_0/BUFG_O]
	set_property -dict [list CONFIG.C_LANE_WIDTH {4} CONFIG.C_REFCLK_FREQUENCY {250.000} CONFIG.Interface_Mode {Streaming}] [get_bd_cells aurora_8b10b_1]
	connect_bd_net [get_bd_pins aurora_8b10b_1/drpclk_in] [get_bd_pins IXIP/clk_50_mhz]
	
	# Instantiate FIFO for Aurora 0 TX
	create_bd_cell -type ip -vlnv xilinx.com:ip:axis_data_fifo:2.0 axis_data_fifo_0
	set_property -dict [list CONFIG.IS_ACLK_ASYNC {1}] [get_bd_cells axis_data_fifo_0]
	connect_bd_intf_net [get_bd_intf_pins ix_axis_interface/M_AXIS_CPU2FPGA_00] [get_bd_intf_pins axis_data_fifo_0/S_AXIS]
	connect_bd_intf_net [get_bd_intf_pins axis_data_fifo_0/M_AXIS] [get_bd_intf_pins aurora_8b10b_0/USER_DATA_S_AXI_TX]
	connect_bd_net [get_bd_pins axis_data_fifo_0/s_axis_aclk] [get_bd_pins IXIP/clk_250_mhz]
	connect_bd_net [get_bd_pins axis_data_fifo_0/m_axis_aclk] [get_bd_pins util_ds_buf_0/BUFG_O]
	create_bd_cell -type ip -vlnv xilinx.com:ip:xlconstant:1.1 xlconstant_0
	connect_bd_net [get_bd_pins xlconstant_0/dout] [get_bd_pins axis_data_fifo_0/s_axis_aresetn]
	
	# Instantiate FIFO for Aurora 0 RX
	create_bd_cell -type ip -vlnv xilinx.com:ip:axis_data_fifo:2.0 axis_data_fifo_1
	set_property -dict [list CONFIG.IS_ACLK_ASYNC {1}] [get_bd_cells axis_data_fifo_1]
	connect_bd_intf_net [get_bd_intf_pins aurora_8b10b_0/USER_DATA_M_AXI_RX] [get_bd_intf_pins axis_data_fifo_1/S_AXIS]
	connect_bd_intf_net [get_bd_intf_pins axis_data_fifo_1/M_AXIS] [get_bd_intf_pins ix_axis_interface/S_AXIS_FPGA2CPU_00]
	connect_bd_net [get_bd_pins axis_data_fifo_1/s_axis_aclk] [get_bd_pins util_ds_buf_0/BUFG_O]
	connect_bd_net [get_bd_pins axis_data_fifo_1/m_axis_aclk] [get_bd_pins IXIP/clk_250_mhz]
	connect_bd_net [get_bd_pins axis_data_fifo_1/s_axis_aresetn] [get_bd_pins xlconstant_0/dout]
	
	# Instantiate FIFO for Aurora 1 TX
	create_bd_cell -type ip -vlnv xilinx.com:ip:axis_data_fifo:2.0 axis_data_fifo_2
	set_property -dict [list CONFIG.IS_ACLK_ASYNC {1}] [get_bd_cells axis_data_fifo_2]
	connect_bd_intf_net [get_bd_intf_pins axis_data_fifo_2/S_AXIS] [get_bd_intf_pins ix_axis_interface/M_AXIS_CPU2FPGA_01]
	connect_bd_intf_net [get_bd_intf_pins axis_data_fifo_2/M_AXIS] [get_bd_intf_pins aurora_8b10b_1/USER_DATA_S_AXI_TX]
	connect_bd_net [get_bd_pins axis_data_fifo_2/s_axis_aresetn] [get_bd_pins xlconstant_0/dout]
	connect_bd_net [get_bd_pins axis_data_fifo_2/s_axis_aclk] [get_bd_pins IXIP/clk_250_mhz]
	connect_bd_net [get_bd_pins axis_data_fifo_2/m_axis_aclk] [get_bd_pins util_ds_buf_0/BUFG_O]
	
	# Instantiate FIFO for Aurora 1 RX
	create_bd_cell -type ip -vlnv xilinx.com:ip:axis_data_fifo:2.0 axis_data_fifo_3
	set_property -dict [list CONFIG.IS_ACLK_ASYNC {1}] [get_bd_cells axis_data_fifo_3]
	connect_bd_intf_net [get_bd_intf_pins axis_data_fifo_3/S_AXIS] [get_bd_intf_pins aurora_8b10b_1/USER_DATA_M_AXI_RX]
	connect_bd_intf_net [get_bd_intf_pins axis_data_fifo_3/M_AXIS] [get_bd_intf_pins ix_axis_interface/S_AXIS_FPGA2CPU_01]
	connect_bd_net [get_bd_pins axis_data_fifo_3/s_axis_aresetn] [get_bd_pins xlconstant_0/dout]
	connect_bd_net [get_bd_pins axis_data_fifo_3/s_axis_aclk] [get_bd_pins util_ds_buf_0/BUFG_O]
	connect_bd_net [get_bd_pins axis_data_fifo_3/m_axis_aclk] [get_bd_pins IXIP/clk_250_mhz]

	regenerate_bd_layout
	
	save_bd_design
}